//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.


#import <UIKit/UIKit.h>

/*!
 * @typedef PCUploadAPIState
 * @brief The various states of an upload process.
 * @constant PCUploadAPIStateInitiatingUpload Upload is being initiated. MD5 calculation, Intent upload call, some file operation etc goes here.
 * @constant PCUploadAPIStateBeginningUpload Upload is about to begin.
 * @constant PCUploadAPIStateUploading Data is being uploaded.
 * @constant PCUploadAPIStateCommitingUpload All data is uploaded, commiting upload. This state comes only for files of size more than 10 MB.
 */
typedef enum {
    PCUploadAPIStateInitiatingUpload,
    PCUploadAPIStateBeginningUpload,
    PCUploadAPIStateUploading,
    PCUploadAPIStateCommitingUpload
} PCUploadAPIState;

@class PCAPI;

@class PCSearchAPIParams;
@class PCThumbnailAPIParams;
@class PCCreateFolderAPIParams;
@class PCFilesAPIParams;
@class PCMetaDataAPIParams;
@class PCFOPSAPIParams;
@class PCDeleteParams;
@class PCGetPlaylistAPIParams;
@class PCGetPlaylistDefinitionAPIParams;
@class PCGetPlaylistContentAPIParams;
@class PCDeletePlaylistAPIParams;
@class PCGetPlaylistItemsAPIParams;
@class PCCreateplaylistAPIParams;
@class PCAddPlaylistItemsAPIParams;
@class PCDeletePlaylistItemAPIParams;
@class PCEditPlaylistAPIParams;
@class PCEditPlaylistDefinitionAPIParams;
@class PCUploadAPIParams;
@class PCGetTrashAPIParams;
@class PCDeleteTrashAPIParams;
@class PCRestoreAPIParams;

@class PCClientCredentials;
@class PCAuthenticationResponse;
@class PCOAuthLoginPageResponse;
@class PCAccountInfo;
@class PCResource;
@class PCMetadata;
@class PCPlaylist;
@class PCAPIQueryResponse;

/*!
 * This class acts as the interface between Verizon API sdk and third party applications. Third party applications can invoke all Verizon API's using public methods exposed in this class.
 */

@interface PCAPISDK : NSObject

/*!
 * Returns shared instance of PCAPISDK class.
 */
+ (PCAPISDK *)sharedInstance;

/*!
 * This method is used when your application is enabled to do a background fetch. Implement application:handleEventsForBackgroundURLSession:completionHandler delegate method in your application delegate class and pass completion handler recieved on to the handleSessionCompletionHandler: method.
 * @param sessionIdentifier The identifier of the URL session, recieved in. application:handleEventsForBackgroundURLSession:completionHandler method.
 * @param completionHandler completionHanlder received in application:handleEventsForBackgroundURLSession:completionHandler method.
 */
- (void)handleCompletionHandler:(void (^)())completionHandler forSession:(NSString *)sessionIdentifier;

/*!
 * Call this method to present OAuth2 login screen. Once authentication is done/failed completion handler gets called. In case of success, authentication credentials are passed in the completion handler. Access token received as a result of login is saved in keychain. This will remain in keychain until user forcefully calls logout.
 * @param clientCredentials details like ClientID, clientSecret, redirectURL, keychainID etc.
 * @param viewController viewController from where OAuth2 screen is to be presented.
 * @param completion This completion block gets called once OAuth2 process is done. If user is successfully signed in, authentication credentials are returned in this block.
 */
- (void)authenticateClient:(PCClientCredentials *)clientCredentials
        fromViewController:(UIViewController *)viewController
      withCompletionHandler:(void (^)(PCAuthenticationResponse *authenticationCredentials, PCOAuthLoginPageResponse *OAuthLoginPageResponse, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to logout user from current session. Access token stored in keychain is also removed. Returns YES if token was successfully removed from keychain.
 */
- (BOOL)logout;

/*!
 * Returns YES if user is user has an access token.
 */
- (BOOL)isUserLoggedIn;

/*!
 * Call this methods to refresh current session. Once session is refreshed authentication credentials are returned with completion block.
 * @param completion This block gets called on the completion of refresh session process. new authenticationCredetials are returned in completion block.
 */
- (PCAPI *)refreshSessionWithCompletionHandler:(void (^)(PCAuthenticationResponse *authenticationCredentials, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to refresh the local cache of files. This method first calls fullview api to get the changes from server, then merge changes with local database.
 * @param completion This block gets called on the completion of full sync operation.
 */
- (void)doFullSyncWithCompletionHandler:(void (^)(PCAPIQueryResponse *response))completion;

/*!
 * Call this method to fetch metadata of a folder.
 * @param params parameters required to fetch metadata of a folder.
 * @param completion This block gets called on the completion of metadata fetch process. A PCMetadata object with metadata details are returned in completion block.
 */
- (PCAPI *)fetchMetadata:(PCMetaDataAPIParams *)params withCompletionHandler:(void (^)(PCMetadata *metadata, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to download thumbnail image of any file.
 * @param params parameters required to fetch a thumbnail image.
 * @param completion This block gets called on the completion of thumbnail fetch process. PCAPIQueryResponse stores the location of downloaded thumbnail.
 * @return A PCAPI object which is downloading the thumbnail image.
 */
- (PCAPI *)downloadThumbnail:(PCThumbnailAPIParams *)params withCompletionHandler:(void (^)(PCAPIQueryResponse *response))completion;

/*!
 * Call this method to download a file. The file is downloaded to a temporary path and the file will be removed once the api which downloaded the file is released. So move the file to a different location in completion callback.
 * @param params parameters required to download a file.
 * @param progress indicates the upload progress of the file.
 * @param completion This block gets called on the completion of a file download. PCAPIQueryResponse stores the location of downloaded file.
 * @return A PCAPI object which is downloading the file.
 */
- (PCAPI *)downloadFile:(PCFilesAPIParams *)params progress:(void(^)(long long totalBytesDownloaded, long long totalBytesToBeDownloaded))progress withCompletionHandler:(void(^)(PCAPIQueryResponse *response))completion;

/*!
 * Call this method to fetch account details.
 * @param completion This block gets called on the completion of account info fetch process. An object of PCAccountInfo which holds account information is returned in completion block.
 */
- (PCAPI *)fetchAccountInfoWithCompletionHandler:(void (^)(PCAccountInfo *accountInfo, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to create a folder at a given path.
 * @param params parameters required to create a folder.
 * @param completion This block gets called on the completion of a folder creation process. Created folder metadata is returned in completion block.
 */
- (PCAPI *)createFolder:(PCCreateFolderAPIParams *)params withCompletionHandler:(void (^)(PCResource *metadata, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to search for any resource in local database.
 * @param params parameters required to perform a search operation.
 * @param completion This block gets called on the completion of search process. An array of NSManagedObjectID is returned in the completion block.
 */
- (void)search:(PCSearchAPIParams *)params withCompletionHandler:(void (^)(NSArray *results, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to move a file/folder from one location to a different location.
 * @param params parameters required to perform move operation.
 * @param completion This block gets called on the completion of a move operation. meta data of 'target folder' is returned in completion block.
 */
- (PCAPI *)move:(PCFOPSAPIParams *)params withCompletionHandler:(void (^)(PCResource *metadata, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to copy a file/folder from one location to a different location.
 * @param params parameters required to perform copy operation.
 * @param completion This block gets called on the completion of a copy operation. meta data of 'target folder' is returned in completion block.
 */
- (PCAPI *)copy:(PCFOPSAPIParams *)params withCompletionHandler:(void (^)(PCResource *metadata, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to rename a file/folder.
 * @param params parameters required to perform rename operation.
 * @param completion This block gets called on the completion of a rename operation. meta data of renamed folder is returned in completion block.
 */
- (PCAPI *)rename:(PCFOPSAPIParams *)params withCompletionHandler:(void (^)(PCResource *metadata, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to delete a file/folder.
 * @param params parameters required to perform delete operation.
 * @param completion This block gets called on the completion of a delete operation.
 */
- (PCAPI *)deletePath:(PCDeleteParams *)params withCompletionHandler:(void (^)(PCAPIQueryResponse *response))completion;


/*!
 * Call this method to get user specific list of playlists.
 * @param params parameters required to fetch playlist.
 * @param completion This block gets called on the completion of a playlist fetch operation. An array of PCPlaylist objects are returned in this block.
 */
- (PCAPI *)getPlaylist:(PCGetPlaylistAPIParams *)params withCompletionHandler:(void (^)(NSArray *playlist, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to get user specific playlist definition.
 * @param params parameters required to fetch specific playlist definition.
 * @param completion This block gets called on the completion of a playlist fetch operation. A PCPlaylist object is returned in this block.
 */
- (PCAPI *)getPlaylistDefinition:(PCGetPlaylistDefinitionAPIParams *)params withCompletionHandler:(void (^)(PCPlaylist *playlist, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to get content of a playlist at the specified path.
 * @param params parameters required to fetch content of a playlist.
 * @param completion This block gets called on the completion of a file retrieve. PCAPIQueryResponse stores the location of retrieved file.
 */
- (PCAPI *)getPlaylistContent:(PCGetPlaylistContentAPIParams *)params withCompletionHandler:(void (^)(PCAPIQueryResponse *response))completion;

/*!
 * Call this method to delete a specific playlist.
 * @param params parameters required to delete a specific playlist.
 * @param completion This block gets called on the completion of a playlist delete operation.
 */
- (PCAPI *)deletePlaylist:(PCDeletePlaylistAPIParams *)params withCompletionHandler:(void(^)(PCAPIQueryResponse *response))completion;

/*!
 * Call this method to create a playlist.
 * @param params parameters required to create a playlist.
 * @param completion This block gets called on the completion of a create playlist operation. A PCPlaylist object is returned in this block.
 */
- (PCAPI *)createPlaylist:(PCCreateplaylistAPIParams *)params withCompletionHandler:(void (^)(PCPlaylist *playlist, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to edit a playlist.
 * @param params parameters required to edit a playlist.
 * @param completion This block gets called on the completion of a rename playlist operation. Renamed PCPlaylist object is returned in this block.
 */
- (PCAPI *)editPlaylist:(PCEditPlaylistAPIParams *)params withCompletionHandler:(void (^)(PCPlaylist *playlist, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to edit a playlist definition.
 * @param params parameters required to update a playlist by sending a new 'playlistDefinition' model.
 * @param completion This block gets called on the completion of a update playlist definition operation. Updated PCPlaylist object is returned in this block.
 */
- (PCAPI *)editPlaylistDefinition:(PCEditPlaylistDefinitionAPIParams *)params withCompletionHandler:(void (^)(PCPlaylist *playlist, PCAPIQueryResponse *response))completion;
/*!
 * Call this method to get details of all the playlist's items for a given playlist.
 * @param params parameters required to fetch playlist items of a given playlist.
 * @param completion This block gets called on the completion of a playlist item fetch operation. An array of PCResource objects are returned in this block.
 */
- (PCAPI *)getPlaylistItems:(PCGetPlaylistItemsAPIParams *)params withCompletionHandler:(void (^)(NSArray *playlistItems, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to delete an item in a given playlist.
 * @param params parameters required to delete an item in a given playlist.
 * @param completion This block gets called on the completion of delete playlist item operation.
 */
- (PCAPI *)deletePlaylistItem:(PCDeletePlaylistItemAPIParams *)params withCompletionHandler:(void (^)(PCAPIQueryResponse *response))completion;

/*!
 * Call this method to add items inside a playlist.
 * @param params parameters required to add items in a playlist.
 * @param completion This block gets called on the completion of a add Items to playlist operation. A NSArray of PCResource objects are returned in this block.
 */
- (PCAPI *)addPlaylistItems:(PCAddPlaylistItemsAPIParams *)params withCompletionHandler:(void (^)(NSArray *playlistItems, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to upload a file to a folder.
 * @param params parameters required to upload the file.
 * @param filePath local path of the file from where file has to be read.
 * @param progress indicates the upload progress of the file.
 * @param completion This block gets called on the completion of the upload process. metadata information of the uploaded file is returned in completion.
 */
- (void)uploadFile:(PCUploadAPIParams *)params readFromPath:(NSString *)filePath progress:(void(^)(PCUploadAPIState uploadState, long long totalBytesUploaded, long long totalBytesToBeUploaded))progress withCompletionHandler:(void(^)(PCResource *metadata, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to fetch all contact details in an account.
 * @param completion This block gets called on the completion of the contact fetch process. An array of PCContact objects are returned in completion.
 */
- (PCAPI *)fetchContactsWithCompletionHandler:(void (^)(NSArray *contacts, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to fetch a list of files and folders that have been deleted from a user repository.
 * @param params parameters required to fetch a list of files and folders that have been deleted from a user repository.
 * @param completion This block gets called on the completion of trash fetch process. An array of PCMetadata objects are returned in completion block.
 */
- (void)fetchTrash:(PCGetTrashAPIParams *)params  withCompletionHandler:(void (^)(NSArray *trashItems, PCAPIQueryResponse *response))completion;

/*!
 * Call this method to delete all the files and folder that are in a deleted state in the repository.
 * @param completion This block gets called on the completion of a delete trash operation.
 */
- (void)deleteTrashWithCompletionHandler:(void (^)(PCAPIQueryResponse *response))completion;

/*!
 * Call this method to restore files or folders that are marked as deleted. When called on a folder, the API creates a new version of the folder and any child folder, and it restores all files in these folders.
 * @param params parameters required to perform restore operation.
 * @param completion This block gets called on the completion of a restore operation.
 */
- (PCAPI *)restore:(PCRestoreAPIParams *)params withCompletionHandler:(void (^)(PCAPIQueryResponse *response))completion;


@end
